/* ------------------------------------------------------------------------
System       : cc65 cross-development environment
Platform     : Apple IIe 128K PRODOS 8
Program      : dloshow(C) Copyright Bill Buckels 2013,2014.
               All rights reserved.

Description  : Apple //e ProDOS 8 DLOGR Slideshow Program

DLOGR formats supported:  DLO raster or DL1 DL2 bsaved pairs.

Written by   : Bill Buckels
Date Written : January 2013
Revision     : 1.0 First Release (Aztec C65)
Date Revised : May 2014
               2.0 Second Release (cc65)

               I really haven't changed this code much since
               writing the Aztec C65 version. All I have really done
               is make it fit with Oliver Schmidt's library routines
               as much as I felt it was necessary.

               Rather than create a modified tgi driver for this demo
               I have pretty-much written this the way a BASIC or an
               ML programmer who knows the Apple II might do, so folks
               can follow along. This is not a brilliant piece of work
               by any means.

Licence      : You may use this program for whatever you wish as long
               as you agree that Bill Buckels has no warranty or
               liability obligations whatsoever from said use.
------------------------------------------------------------------------ */

#ifndef __APPLE2__
#define __APPLE2__
#endif

#include <stdio.h>
#include <errno.h>
#include "_file.h"
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <conio.h>
#include <peekpoke.h>

/* http://www.unix.com/man-page/posix/3/opendir/ */
/* files in current directory */
#include <dirent.h>

#include <apple2enh.h>

/* base addresses for primary text page */
/* also the base addresses for the 48 scanline pairs */
/* for lores graphics mode 40 x 48 x 16 colors */
unsigned textbase[24]={
    0x0400,
    0x0480,
    0x0500,
    0x0580,
    0x0600,
    0x0680,
    0x0700,
    0x0780,
    0x0428,
    0x04A8,
    0x0528,
    0x05A8,
    0x0628,
    0x06A8,
    0x0728,
    0x07A8,
    0x0450,
    0x04D0,
    0x0550,
    0x05D0,
    0x0650,
    0x06D0,
    0x0750,
    0x07D0};


void dloreson(void)
{
	asm("sta  $c050"); /* GRAPHICS */
    asm("sta  $c052"); /* GRAPHICS ONLY, NOT MIXED */
    asm("sta  $c054"); /* START IN MAIN MEM */
    asm("sta  $c056"); /* LO-RES */
    asm("sta  $c05e"); /* DOUBLE */
}


void dloresclear()
{
	int idx;
	for (idx = 0; idx < 24; idx++) {
	    asm("sta  $c055"); /* AUX MEM */
	    memset ((char *)textbase[idx], 0, 40);
	    asm("sta  $c054"); /* MAIN MEM */
	    memset ((char *)textbase[idx], 0, 40);
	}

}

void dloresoff(void)
{
    asm("sta  $c051"); /* TEXT */
    asm("sta  $c05f"); /* SINGLE */
    asm("sta  $c054"); /* PAGE ONE */
}

/* The following loads two non-compressed
   double lo-res graphics image formats:

   Single File Raster Format - DLO
   Bsaved Image Format file pairs - DL1 and DL2

*/
char lodebuf[1920];

int dlodelo(char *name)
{

	FILE *fp;

	int y, status=-2;
	int c, fl = 1016, height, packet, jdx, idx;
	char tempchar[2], name1[20], name2[20], *ptr1, *ptr2;

    asm("sta  $c054"); /* MAIN MEM - safety play */

	jdx = 999;
	for (idx = 0; name[idx] != 0; idx++) {
		name1[idx] = name2[idx] = name[idx];
		if (name[idx] == '.') {
			name1[idx] = name2[idx] = 0;
			jdx = idx;
			break;
		}
	}
   	if (jdx == 999) return status;

    /* try to open a BASIC image pair */
  	strcat(name2,".DL1");

	if ((fp = fopen(name2,"rb"))== NULL) {
		/* if we can't open an image pair then try for a
		   raster file... */
	   	strcat(name1,".DLO");
		if ((fp = fopen(name1,"rb"))== NULL) return -1;
		fl = 1922;
		ptr1 = (char *)&lodebuf[0];
		ptr2 = (char *)&lodebuf[40];

	}
	else {
	   	/* if we opened a DL1 then get ready to open a DL2 */
	   	strcat(name1,".DL2");
	}

   	switch(fl) {
	   	case 1922:
	        /* is it a DLO ? - read the 2 byte header */
	        c = fread((char *)&tempchar[0],1,2,fp);
			if (c!=2) {
				fclose(fp);
				break;
			}
			packet= (int)tempchar[0];
			height= (int)tempchar[1];
			if (height != 24 || packet != 80) {
				fclose(fp);
				break;
			}

			c = fread(lodebuf,1,1920,fp);
			fclose(fp);
			if (c!=1920)break;
            status = 0;

			for(y=0,idx=0;y<height;y++,idx+=80) {
				asm("sta  $c055"); /* AUX MEM */
				memcpy((char *)textbase[y],(char *)&ptr1[idx],40);
				asm("sta  $c054"); /* MAIN MEM */
				memcpy((char *)textbase[y],(char *)&ptr2[idx],40);

			}
			break;

	   case 1016:
	       /* is it a bsaved image ? */
	       c = fread(lodebuf,1,1016,fp);
	       fclose(fp);
	       if (c != 1016)break;
		   asm("sta  $c055"); /* AUX MEM */
		   memcpy((char *)0x0400,lodebuf,1016);
		   asm("sta  $c054"); /* MAIN MEM */
		   if ((fp = fopen(name1,"rb"))== NULL) {
			   status = -1;
			   break;
		   }
		   c = fread(lodebuf,1,1016,fp);
		   fclose(fp);
	       if (c != 1016)break;
           memcpy((char *)0x0400,lodebuf,1016);
           status=0;
		   break;
   }

   return status;
}

/* 1MHZ timing loop settings - increase for faster machines */
#define YTIME 60L
#define XTIME 32L
char wait(unsigned duration)
{
	char c = 0;
	long y, x;

	while (duration > 0) {
		for (y=0;y<YTIME;y++) {
			for (x=0;x<XTIME;x++) {
				if (kbhit()) {
					c = cgetc();
					/* clear keyboard buffer */
					while (kbhit())cgetc();
					return c;
				}
			}
		}
		duration--;
	}
	return c;
}

/* the following function creates a list of the double lo-res
   files in the current directory, in Apple sequential text format. */
FILE *makepiclist()
{
	FILE *fp = NULL;
	int err, cnt=0, idx, jdx;
  	DIR *dir;
	struct dirent *dp;
	char ch;

	extern unsigned char _filetype;
	extern unsigned _auxtype;

    /* read files in current directory */
	if ((dir = opendir (".")) == NULL) return NULL;

    /* remove piclist if it already exists */
    if((fp=fopen("PICLIST","r"))!=NULL) {
		fclose(fp);
		remove("PICLIST");
	}

    /* set file to sequential text */
	_filetype = 0x04;
	_auxtype = 0x0000;

	if((fp=fopen("PICLIST","w"))==NULL) {
		closedir(dir);
		return NULL;
	}

    /* write the list to disk */
	while ((dp = readdir (dir)) != NULL) {
		 /* binary files only - load address 0400 */
		 if (dp->d_type != 0x06 || dp->d_auxtype != 0x0400) continue;
		 /* get extension */
		 jdx = 999;
		 for (idx = 0; dp->d_name[idx] != 0; idx++) {
			 if (dp->d_name[idx] == '.') {
				 jdx = idx+1; break;
			 }
		 }
		 /* extension must be .DLO or .DL1 */
		 if (jdx == 999) continue;
		 ch = toupper(dp->d_name[jdx]);
		 if (ch != 'D') continue;
		 ch = toupper(dp->d_name[jdx+1]);
		 if (ch != 'L') continue;
		 ch = toupper(dp->d_name[jdx+2]);
		 if (ch != 'O' && ch != '1') continue;
		 /* filesize must match */
		 if (ch == '1' && dp->d_size != 1016) continue;
		 if (ch == 'O' && dp->d_size != 1922) continue;
		 /* if out of disk space quit writing to piclist */
		 err = fprintf(fp,"%s%c",dp->d_name, (char)13);
		 if (err < 0) break;
		 cnt++;
	}

    fclose(fp);
    closedir(dir);

    if (cnt == 0) {
		/* if no files found or no filenames written remove empty piclist */
		remove("PICLIST");
		fp = NULL;
	}
	else {
		/* otherwise open and return the piclist */
		fp = fopen("PICLIST","r");
	}

	return fp;
}

/* fgets for Apple II text files - read native apple II text files properly!
   the cc65 version doesn't! Which means that you need to use unix text files
   unless you roll your own fgets... */
char* __fastcall__ fgets (char* s, unsigned size, register FILE* f)
{
    register char* p = s;
    unsigned i;
    int c;

    if (size == 0) {
        /* Invalid size */
        return (char*) _seterrno (EINVAL);
    }

    /* Read input */
    i = 0;
    while (--size) {

        /* Get next character */
        if ((c = fgetc (f)) == EOF) {
            /* Error or EOF */
            if ((f->f_flags & _FERROR) != 0 || i == 0) {
                /* ERROR or EOF on first char */
                *p = '\0';
                return 0;
            } else {
                /* EOF with data already read */
                break;
            }
        }

        /* One char more */
        *p = c;
        ++p;
        ++i;

        /* Stop at end of Apple II Text File line */
        if ((char)c == (char)13) {
            break;
        }
    }

    /* Terminate the string */
    *p = '\0';

    /* Done */
    return s;
}

int main(void)
{
  	FILE *fp;
   	int status = 0;
   	char picname[66];

    /* try to create a piclist if it does not exist */
    if((fp=fopen("PICLIST","r"))==NULL) {

	    fp = makepiclist();
		if (fp == NULL) {
			printf("Can't open PICLIST! Press any key...");
			cgetc();
			return 0;
		}
    }

    /* initialize 80 column card */
   	videomode(VIDEOMODE_80COL);
   	clrscr();

   	/* turn-on double lo-res and clear the dlgr screen */
   	dloreson();
   	dloresclear();

    /* display each dlgr pic for 5 seconds or advance on
       a key press. If ESC is pressed, exit */
    for (;;) {
		while (fgets(picname,66,fp) != NULL) {
			/* load picture - ignore errors */
			dlodelo(picname);
			POKE(36,0); POKE(37,0); /* HOME MESSY TEXT CURSOR */

			while (kbhit())cgetc();
        	/* advance on timeout or advance on a keypress */
	   		if (wait(5)==27) {
				/* if ESC is pressed, exit */
				status = -1;
				break;
			}
		}
		fclose(fp);
		if (status != 0) break;
		/* no rewind in cc65 yet, so we must
		   close and re-open the piclist when we
		   reach the end */
		if((fp=fopen("PICLIST","r"))==NULL) break;
	}

	dloresclear();
    dloresoff();
  	clrscr();

    return 0;
}
